@@ -1,5 +1,5 @@
 ;; frames.jl -- handle window framing
-;; $Id: frames.jl,v 1.76 2000/11/03 02:49:40 jsh Exp $
+;; $Id: frames.jl,v 1.82 2001/01/27 02:36:40 jsh Exp $
 
 ;; Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
 
@@ -64,7 +64,8 @@
 	  sawfish.wm.events
 	  sawfish.wm.gaol
 	  sawfish.wm.session.init
-	  sawfish.wm.workspace)
+	  sawfish.wm.workspace
+	  sawfish.wm.state.maximize)
 
   ;; Commentary:
 
@@ -117,29 +118,7 @@
 
 ;;; variables etc
 
-  (defvar frame-part-classes
-    '((title . ((cursor . hand2)
-		(keymap . title-keymap)))
-      (menu-button . ((keymap . menu-button-keymap)))
-      (close-button . ((keymap . close-button-keymap)))
-      (iconify-button . ((keymap . iconify-button-keymap)))
-      (maximize-button . ((keymap . maximize-button-keymap)))
-      (top-border . ((cursor . top_side)
-		     (keymap . border-keymap)))
-      (left-border . ((cursor . left_side)
-		      (keymap . border-keymap)))
-      (right-border . ((cursor . right_side)
-		       (keymap . border-keymap)))
-      (bottom-border . ((cursor . bottom_side)
-			(keymap . border-keymap)))
-      (top-left-corner . ((cursor . top_left_corner)
-			  (keymap . border-keymap)))
-      (top-right-corner . ((cursor . top_right_corner)
-			   (keymap . border-keymap)))
-      (bottom-left-corner . ((cursor . bottom_left_corner)
-			     (keymap . border-keymap)))
-      (bottom-right-corner . ((cursor . bottom_right_corner)
-			      (keymap . border-keymap))))
+  (defvar frame-part-classes nil
     "Alist of (CLASS . ALIST) associating classes of frame parts with state
 they inherit.")
 
@@ -151,11 +130,12 @@ that overrides settings set elsewhere.")
     "Frame definition used for unframed windows.")
 
   (defcustom default-frame-style nil
-    "Default frame style (theme)."
+    "Default frame style:"
     :type frame-style
+    :widget-flags (expand-vertically)
     :user-level novice
     :group appearance
-    :after-set (lambda () (after-setting-frame-option)))
+    :after-set (lambda () (after-setting-default-frame)))
 
   (defcustom reload-themes-when-changed t
     "Automatically reload themes when they are updated."
@@ -171,13 +151,14 @@ that overrides settings set elsewhere.")
       (shaded-transient . shaped-transient)
       (icon . shaped-transient)
       (dock . icon))
-    "Frame type fallbacks."
+    "Frame type fallbacks:"
     :tooltip "Associate frame types with type to try if the theme doesn't \
 implement the requested type."
     :type (alist ((symbol default shaped transient
 			  shaped-transient icon doc) "From")
 		 ((symbol default shaped transient
 			  shaped-transient icon doc) "To"))
+    :widget-flags (expand-vertically framed)
     :group appearance
     :user-level expert
     :after-set (lambda () (after-setting-frame-option)))
@@ -233,6 +214,7 @@ deciding which frame type to ask a theme to generate.")
     "Default font: \\w"
     :group appearance
     :type font
+    :widget-flags (expand-horizontally)
     :user-level novice
     :after-set (lambda () (after-setting-frame-option)))
 
@@ -336,6 +318,10 @@ deciding which frame type to ask a theme to generate.")
 
   (define after-setting-frame-option reframe-all-windows)
 
+  (define (after-setting-default-frame)
+    (check-frame-availability default-frame-style)
+    (after-setting-frame-option))
+
   (define (rebuild-frames-with-style style)
     (map-windows (lambda (w)
 		   (when (eq (window-get w 'current-frame-style) style)
@@ -433,9 +419,11 @@ deciding which frame type to ask a theme to generate.")
       (catch 'out
 	(mapc (lambda (cell)
 		(when (string-match (car cell) dir)
-		  (throw 'out (expand-last-match (if get-name
-						     (nth 2 cell)
-						   (nth 1 cell))))))
+		  (let ((full (expand-last-match (nth 1 cell))))
+		    (when (file-directory-p full)
+		      (throw 'out (if get-name
+				      (expand-last-match (nth 2 cell))
+				    full))))))
 	      theme-suffix-regexps)
 	nil)))
 
@@ -584,9 +572,40 @@ deciding which frame type to ask a theme to generate.")
 	  (set-frame-part-value class 'keymap map-name)))
       ok-to-bind))
 
+  (define ((cursor-for-frame-part part) w)
+    (if (frame-part-movable-p w part)
+	(case part
+	  ((top-border) 'top_side)
+	  ((bottom-border) 'bottom_side)
+	  ((left-border) 'left_side)
+	  ((right-border) 'right_side)
+	  ((top-left-corner) 'top_left_corner)
+	  ((top-right-corner) 'top_right_corner)
+	  ((bottom-left-corner) 'bottom_left_corner)
+	  ((bottom-right-corner) 'bottom_right_corner)
+	  ((title) 'hand2))
+      nil))
+
 
 ;;; initialisation
 
+  (define-frame-class 'menu-button '((keymap . menu-button-keymap)))
+  (define-frame-class 'close-button '((keymap . close-button-keymap)))
+  (define-frame-class 'iconify-button '((keymap . iconify-button-keymap)))
+  (define-frame-class 'maximize-button '((keymap . maximize-button-keymap)))
+  (define-frame-class 'shade-button '((keymap . shade-button-keymap)))
+
+  (define-frame-class 'title `((keymap . title-keymap)
+			       (cursor . ,(cursor-for-frame-part 'title))))
+
+  (mapc (lambda (x)
+	  (define-frame-class x `((cursor . ,(cursor-for-frame-part x))
+				  (keymap . border-keymap))))
+	'(top-border bottom-border
+	  left-border right-border
+	  top-left-corner top-right-corner
+	  bottom-left-corner bottom-right-corner))
+
   (gaol-add add-frame-style reframe-window rebuild-frames-with-style
 	    reframe-windows-with-style reframe-all-windows window-type
 	    def-frame-class define-frame-class after-setting-frame-option
